/*
  smash, a chess engine
  Copyright (c) 2003-2006  Maurizio Sambati
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  
  See LICENSE file for more info.

  written by Maurizio Sambati <sambati@cli.di.unipi.it>
  http://www.cli.di.unipi.it/~sambati
*/
#ifndef _EVAL_H_
#define _EVAL_H_

/*
  Evaluation unit

  Created in 02/07/2004
  Maurizio
*/

#include <climits>
#include "smash.h"

typedef int score_t;


// C O N S T A N T S

// piece scores
const int PAWN_VALUE = 100;
const int KNIGHT_VALUE = 300;
const int BISHOP_VALUE = 320;
const int ROOK_VALUE = 500;
const int QUEEN_VALUE = 900;
const int KING_VALUE = 1000;
const extern int PIECE_VALUE[];
const extern char positional_score[PIECES_NUM][2][MAXPOS];

// score limits
const int SCORE_MAX = SHRT_MAX-2;
const int SCORE_MIN = -SCORE_MAX;
const int SCORE_NULL = SCORE_MIN - 2; // invalid position score
const int SCORE_ONEVALUATION = SCORE_MIN - 1; // ABDADA to let thread knows that someone is working on a position
const int SCORE_DRAW = 0;
const int SCORE_MATEHI = SCORE_MAX - 500;
const int SCORE_MATELO = 500 - SCORE_MAX;

const int OPENING = 0;
const int ENDING = 1;

// F U N C T I O N S

void InitEval();

// evaluate board
class Board;
score_t Eval( const Board &b );


// I N L I N E   F U N C T I O N S

/*
  return score tuned by phase
*/
inline score_t MergeScore( score_t opening, score_t ending, int phase )
{
  return ( opening * (256-phase) + ending * phase ) / 256;
}



#endif
