/*
  smash, a chess engine
  Copyright (c) 2003-2006  Maurizio Sambati
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  
  See LICENSE file for more info.

  written by Maurizio Sambati <sambati@cli.di.unipi.it>
  http://www.cli.di.unipi.it/~sambati
*/
#include <cstdio>
#include <cstring>
#include <cerrno>

#if defined(_WIN32) || defined(_WIN64)
#include <windows.h>
#else
#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>
#endif


#include "input.h"
#include "smash.h"
#include "smp.h"

using namespace std;

/*
  Check for input, return true if input is available

  NOTE: Copied from fruit source...
*/
bool InputAvailable()
{
#if defined(_WIN32) || defined(_WIN64)
  static bool init = false, is_pipe;
  static HANDLE stdin_h;
  DWORD val;

  if( stdin->_cnt > 0 ) {
    return true;
  }

  if( !init ) {
    init = true;
    stdin_h = GetStdHandle( STD_INPUT_HANDLE );
    is_pipe = !GetConsoleMode( stdin_h, &val );
    if( !is_pipe ) {
      SetConsoleMode( stdin_h, val & ~(ENABLE_MOUSE_INPUT|ENABLE_WINDOW_INPUT) );
      FlushConsoleInputBuffer( stdin_h );
    }
  }

  if( is_pipe ) {
    if( !PeekNamedPipe( stdin_h, NULL, 0, NULL, &val, NULL ) ) {
      return true;
    }
    return val > 0;
  } else {
    GetNumberOfConsoleInputEvents( stdin_h, &val );
    return val > 1;
  }
  
#else
  int ret;
  fd_set rfds;
  struct timeval tv;

  FD_ZERO( &rfds );
  FD_SET( fileno( stdin ), &rfds );

  // don't wait
  tv.tv_sec = 0;
  tv.tv_usec = 0;

  ret = select( fileno( stdin ) + 1, &rfds, NULL, NULL, &tv );
  if( ret == -1 && errno != EINTR ) {
    exit( 1 );
  }

  return ( ret > 0 );
#endif
}
