/*
  smash, a chess engine
  Copyright (c) 2003-2006  Maurizio Sambati
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  
  See LICENSE file for more info.

  written by Maurizio Sambati <sambati@cli.di.unipi.it>
  http://www.cli.di.unipi.it/~sambati
*/
#include <string>

#include "move.h"
#include "board.h"

using namespace std;

/*
  Translate move to string in long algebraic notation
*/
string Move2Str( move_t m )
{
  if( m == NULLMOVE ) {
    return "0000";
  }
  assert( Src( m ) < MAXPOS );
  assert( Dst( m ) < MAXPOS );
  string tmp = square2str[Src( m )];
  tmp += square2str[Dst( m )];

  /* if the move is a pawn promotion append the
     piece we want to promote to */
  if( IsPromote( m ) ) {
    switch( Promote( m ) ) {
    case KNIGHT:
      tmp += "n";
      break;

    case BISHOP:
      tmp += "b";
      break;

    case ROOK:
      tmp += "r";
      break;

    case QUEEN:
      tmp += "q";
      break;

    default:
      assert( 0 );
    }
  }

  return tmp;
}

/*
  Return move binary representation of the string
*/
move_t Str2Move( const Board &board, const string &str )
{
  string str_src( str, 0, 2 );
  string str_dst( str, 2, 2 );

  square_t src = -1, dst = -1;
  for( int i = 0; i < MAXPOS; i++ ) {
    if( str_src == square2str[i] ) {
      src = i;
    }
    if( str_dst == square2str[i] ) {
      dst = i;
    }
  }

  if( src == -1 || dst == -1 ) {
    assert( 0 );
  }
      
  move_t move = MakeMove( src, dst );
  if( board[dst] != EMPTY ) {
    move = AddCapture( move, GetPiece( board[dst] ) );
  }

  piece_t piece = GetPiece( board[src] );
  if( piece == PAWN && abs( src - dst )/MAXCOL == 2 ) {
    move |= MOVE_DOUBLE_PAWN_PUSH;
  } else if( piece == KING && abs( src - dst ) == 2 ) {
    move |= MOVE_CASTLING;
  } else if( piece == PAWN && ( Row( dst ) == 0 || Row( dst ) == MAXROW-1 ) ) {
    piece_t promote_piece;

    switch( str[4] ) {
    case 'q':
      promote_piece = QUEEN; break;
    case 'n':
      promote_piece = KNIGHT; break;
    case 'r':
      promote_piece = ROOK; break;
    case 'b':
      promote_piece = BISHOP; break;
    default:
      assert( 0 );
    }
    
    move = AddPromote( move, promote_piece );
  } else if( piece == PAWN && dst == board.en_passant ) {
    move = AddCapture( move, PAWN );
    move |= MOVE_EN_PASSANT;
  }

  return move;
}
