/*
  smash, a chess engine
  Copyright (c) 2003-2006  Maurizio Sambati
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  
  See LICENSE file for more info.

  written by Maurizio Sambati <sambati@cli.di.unipi.it>
  http://www.cli.di.unipi.it/~sambati
*/
#ifndef _MOVE_H_
#define _MOVE_H_

/*
  Move type

  Created in 03/07/2004
  Maurizio
*/

#include <string>
#include "smash.h"

typedef unsigned int move_t;

const move_t MOVE_SQUARE_MASK    = 0x7F; // 7 bit source, 7 bit dest
const move_t MOVE_CAPTURE_MASK   = 0x07; // 3 bit capture
const move_t MOVE_PROMOTE_MASK   = 0x07; // 3 bit promoting
const move_t MOVE_DEST_SHIFT = 7;
const move_t MOVE_CAPTURE_SHIFT = 14;
const move_t MOVE_PROMOTE_SHIFT = 17;
const move_t MOVE_CASTLING         = 0x80000000; // 1 bit castling
const move_t MOVE_DOUBLE_PAWN_PUSH = 0x40000000; // 1 bit pawn double move
const move_t MOVE_EN_PASSANT       = 0x20000000; // 1 bit en passant

const move_t NULLMOVE = 0;

inline square_t Src( move_t m )
{
  return ( square_t )( m & MOVE_SQUARE_MASK );
}

inline square_t Dst( move_t m )
{
  return ( square_t )( (m>>MOVE_DEST_SHIFT) & MOVE_SQUARE_MASK );
}

inline piece_t Capture( move_t m )
{
  return ( piece_t )( (m>>MOVE_CAPTURE_SHIFT) & MOVE_CAPTURE_MASK );
}

inline bool IsCapture( move_t m )
{
  return (m>>MOVE_CAPTURE_SHIFT) & MOVE_CAPTURE_MASK;
}

inline piece_t Promote( move_t m )
{
  return ( piece_t )( (m>>MOVE_PROMOTE_SHIFT) & MOVE_PROMOTE_MASK );
}

inline bool IsPromote( move_t m )
{
  return (m>>MOVE_PROMOTE_SHIFT) & MOVE_PROMOTE_MASK;
}

inline bool IsEnPassant( move_t m )
{
  return m & MOVE_EN_PASSANT;
}

inline bool IsDoublePawnPush( move_t m )
{
  return m & MOVE_DOUBLE_PAWN_PUSH;
}

inline move_t MakeMove( square_t src, square_t dst )
{
  return ( move_t )src | ( ( move_t )dst << MOVE_DEST_SHIFT );
}

inline move_t AddMove( move_t m, square_t src, square_t dst )
{
  return m | ( move_t )src | ( ( move_t )dst << MOVE_DEST_SHIFT );
}

inline move_t MakeCapture( square_t src, square_t dst, piece_t capture )
{
  return ( move_t )src | ( ( move_t )dst << MOVE_DEST_SHIFT ) | ( ( move_t )capture << MOVE_CAPTURE_SHIFT );
}

inline move_t AddCapture( move_t m, piece_t capture )
{
  return m | ( ( move_t )capture << MOVE_CAPTURE_SHIFT );
}

inline move_t MakePromote( square_t src, square_t dst, piece_t promote )
{
  return ( move_t )src | ( ( move_t )dst << MOVE_DEST_SHIFT ) | ( ( move_t )promote << MOVE_PROMOTE_SHIFT );
}

inline move_t AddPromote( move_t m, piece_t promote )
{
  return m | ( ( move_t )promote << MOVE_PROMOTE_SHIFT );
}

inline bool IsCastling( move_t m )
{
  return m & MOVE_CASTLING;
}

class Board;
std::string Move2Str( move_t m );
move_t Str2Move( const Board &board, const std::string &move );

#endif
