/*
  smash, a chess engine
  Copyright (c) 2003-2006  Maurizio Sambati
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  
  See LICENSE file for more info.

  written by Maurizio Sambati <sambati@cli.di.unipi.it>
  http://www.cli.di.unipi.it/~sambati
*/
/*
  Move generation unit

  Created in 28/06/2004
  Maurizio
*/

#include "movegen.h"
#include "bitboard.h"

using namespace std;

bitboard_t pawn_captures[PLAYERS][MAXPOS];
bitboard_t knight_moves[MAXPOS];
bitboard_t bishop_moves[MAXPOS][2][256];
bitboard_t rook_moves[MAXPOS][2][256];
bitboard_t king_moves[MAXPOS];

bool bb_init = false;

static void InitPawnCaptures();
static void InitKnightMoves();
static void InitBishopMoves();
static void InitRookMoves();
static void InitKingMoves();



// I N I T I A L I Z A T I O N

void InitMoveGen()
{
  InitPawnCaptures();
  InitKnightMoves();
  InitBishopMoves();
  InitRookMoves();
  InitKingMoves();
  bb_init = true;
}

void InitPawnCaptures()
{
  for( square_t i = 0; i < MAXCOL; i++ ) {
    pawn_captures[BLACK][i] = NULLBOARD;
  }
  
  pawn_captures[WHITE][CR2Square( 0, 0 )] = bb_square[CR2Square( 1, 1 )];
  for( square_t x = 1; x < MAXCOL-1; ++x ) {
    pawn_captures[WHITE][CR2Square( x, 0 )] = bb_square[CR2Square( x+1, 1 )] | bb_square[CR2Square( x-1, 1 )];
  }
  pawn_captures[WHITE][CR2Square( MAXCOL-1, 0 )] = bb_square[CR2Square( MAXCOL-2, 1 )];
  
  for( square_t y = 1; y < MAXROW-1; ++y ) {
    pawn_captures[WHITE][CR2Square( 0, y )] = bb_square[CR2Square( 1, y+1 )];
    pawn_captures[BLACK][CR2Square( 0, y )] = bb_square[CR2Square( 1, y-1 )];
    for( square_t x = 1; x < MAXCOL-1; ++x ) {
      square_t sq = CR2Square( x, y );
      pawn_captures[WHITE][sq] = bb_square[CR2Square( x-1, y+1 )] | bb_square[CR2Square( x+1, y+1 )];
      pawn_captures[BLACK][sq] = bb_square[CR2Square( x-1, y-1 )] | bb_square[CR2Square( x+1, y-1 )];
    }
    pawn_captures[WHITE][CR2Square( MAXCOL-1, y )] = bb_square[CR2Square( MAXCOL-2, y+1 )];
    pawn_captures[BLACK][CR2Square( MAXCOL-1, y )] = bb_square[CR2Square( MAXCOL-2, y-1 )];
  }

  for( square_t i = CR2Square( 0, MAXROW-1 ); i < MAXPOS; ++i ) {
    pawn_captures[WHITE][i] = NULLBOARD;
  }

  pawn_captures[BLACK][CR2Square( 0, MAXROW-1 )] = bb_square[CR2Square( 1, MAXROW-2 )];
  for( square_t x = 1; x < MAXCOL-1; ++x ) {
    pawn_captures[BLACK][CR2Square( x, MAXROW-1 )] = bb_square[CR2Square( x+1, MAXROW-2 )] | bb_square[CR2Square( x-1, MAXROW-2 )];
  }
  pawn_captures[BLACK][CR2Square( MAXCOL-1, MAXROW-1 )] = bb_square[CR2Square( MAXCOL-2, MAXROW-2 )];
}

void InitKnightMoves()
{
  for( square_t x = 0; x < MAXCOL; ++x ) {
    for( square_t y = 0; y < MAXROW; ++y ) {
      square_t sq = CR2Square( x, y );
      knight_moves[sq] = NULLBOARD;
      if( y >= 1 ) {
	if( x >= 2 ) {
	  knight_moves[sq] |= bb_square[CR2Square( x - 2, y - 1 )];
	}
	if( x < MAXCOL-2 ) {
	  knight_moves[sq] |= bb_square[CR2Square( x + 2, y - 1 )];
	}
      }
      if( y >= 2 ) {
	if( x >= 1 ) {
	  knight_moves[sq] |= bb_square[CR2Square( x - 1, y - 2 )];
	}
	if( x < MAXCOL-1 ) {
	  knight_moves[sq] |= bb_square[CR2Square( x + 1, y - 2 )];
	}
      }
      if( y < MAXROW-1 ) {
	if( x >= 2 ) {
	  knight_moves[sq] |= bb_square[CR2Square( x - 2, y + 1 )];
	}
	if( x < MAXCOL-2 ) {
	  knight_moves[sq] |= bb_square[CR2Square( x + 2, y + 1 )];
	}
      }
      if( y < MAXROW-2 ) {
	if( x >= 1 ) {
	  knight_moves[sq] |= bb_square[CR2Square( x - 1, y + 2 )];
	}
	if( x < MAXCOL-1 ) {
	  knight_moves[sq] |= bb_square[CR2Square( x + 1, y + 2 )];
	}
      }
    }
  }
}

void InitBishopMoves()
{
  for( square_t pos = 0; pos < MAXPOS; ++pos ) {
    for( int perm = 0; perm < 256; ++perm ) {
      bishop_moves[pos][0][perm] = NULLBOARD;
      bishop_moves[pos][1][perm] = NULLBOARD;

#define R45_POS_IN_PERM( x, y ) \
( static_cast<int>( ( bb_square[rot45_lookup[x]] >> rot45_shift[x] ) & rot45_and[x] ) & y )
#define R315_POS_IN_PERM( x, y ) \
( static_cast<int>( ( bb_square[rot315_lookup[x]] >> rot315_shift[x] ) & rot315_and[x] ) & y )
      
      if( R45_POS_IN_PERM( pos, perm ) ) {
	square_t x = Column( pos );
	square_t y = Row( pos );
	bool capt = false;

	while( x > 0 && y > 0 && !capt ) {
	  x--; y--;
	  bishop_moves[pos][0][perm] |= bb_square[CR2Square( x, y )];
	  capt = R45_POS_IN_PERM( CR2Square( x, y ), perm );
	}
	
	x = Column( pos );
	y = Row( pos );
	capt = false;
	while( x < MAXCOL-1 && y < MAXROW-1 && !capt ) {
	  x++;
	  y++;
	  bishop_moves[pos][0][perm] |= bb_square[CR2Square( x, y )];
	  capt = R45_POS_IN_PERM( CR2Square( x, y ), perm );
	}

	// copy in the same position without the piece in pos:
	// needed by Board::attacks()!!
	bishop_moves[pos][0][perm ^ ( ( bb_square[rot45_lookup[pos]] >> rot45_shift[pos] ) & rot45_and[pos] )]
	  = bishop_moves[pos][0][perm];
      }
      
      if( R315_POS_IN_PERM( pos, perm ) ) {
	square_t x = Column( pos );
	square_t y = Row( pos );
	bool capt = false;
	while( x > 0 && y < MAXROW-1 && !capt ) {
	  x--; y++;
	  bishop_moves[pos][1][perm] |= bb_square[CR2Square( x, y )];
	  capt = R315_POS_IN_PERM( CR2Square( x, y ), perm );
	}

	x = Column( pos );
	y = Row( pos );
	capt = false;
	while( x < MAXCOL-1 && y > 0 && !capt ) {
	  x++; y--;
	  bishop_moves[pos][1][perm] |= bb_square[CR2Square( x, y )];
	  capt = R315_POS_IN_PERM( CR2Square( x, y ), perm );
	}

	// copy in the same position without the piece in pos:
	// needed by Board::attacks()!!
	bishop_moves[pos][1][perm ^ ( ( bb_square[rot315_lookup[pos]] >> rot315_shift[pos] ) & rot315_and[pos] )]
	  = bishop_moves[pos][1][perm];
      }
    }
  }
}
		    
void InitRookMoves()
{
  for( square_t pos = 0; pos < MAXPOS; ++pos ) {
    for( int perm = 0; perm < 256; ++perm ) {
      rook_moves[pos][0][perm] = NULLBOARD;
      rook_moves[pos][1][perm] = NULLBOARD;

      square_t x = Column( pos );
      square_t y = Row( pos );
      if( (1<<x) & perm ) {
	square_t x1 = x;
	bool capt = false;
	while( x1 > 0 && !capt ) {
	  --x1;
	  rook_moves[pos][0][perm] |= bb_square[CR2Square( x1, y )];
	  capt = (1<<x1) & perm;
	}
	x1 = x;
	capt = false;
	while( x1 < MAXCOL-1 && !capt ) {
	  ++x1;
	  rook_moves[pos][0][perm] |= bb_square[CR2Square( x1, y )];
	  capt = (1<<x1) & perm;
	}

	// copy in the same position without the piece in pos:
	// needed by Board::attacks()!!
	rook_moves[pos][0][perm^(1<<x)] = rook_moves[pos][0][perm];
      }
      if( ( 1<<Column( rot90_lookup[pos] ) ) & perm ) {
	square_t y1 = y;
	bool capt = false;
	while( y1 > 0 && !capt ) {
	  --y1;
	  rook_moves[pos][1][perm] |= bb_square[CR2Square( x, y1 )];
	  capt = ( 1<<Column( rot90_lookup[CR2Square( x, y1 )] ) ) & perm;
	}
	y1 = y;
	capt = false;
	while( y1 < MAXROW-1 && !capt ) {
	  ++y1;
	  rook_moves[pos][1][perm] |= bb_square[CR2Square( x, y1 )];
	  capt = ( 1<<Column( rot90_lookup[CR2Square( x, y1 )] ) ) & perm;	  
	}

	// copy in the same position without the piece in pos:
	// needed by Board::attacks()!!
	rook_moves[pos][1][perm^( 1<<Column( rot90_lookup[pos] ) )] = rook_moves[pos][1][perm];
      }
    }
  }
}

void InitKingMoves()
{
  for( square_t pos = 0; pos < MAXPOS; ++pos ) {
    king_moves[pos] = NULLBOARD;
    
    square_t x = Column( pos );
    square_t y = Row( pos );

    if( x > 0 ) {
      king_moves[pos] |= bb_square[CR2Square( x-1, y )];
      if( y > 0 ) {
	king_moves[pos] |= bb_square[CR2Square( x-1, y-1 )];
      }
      if( y < MAXROW-1 ) {
	king_moves[pos] |= bb_square[CR2Square( x-1, y+1 )];
      }
    }
    if( x < MAXCOL-1 ) {
      king_moves[pos] |= bb_square[CR2Square( x+1, y )];
      if( y > 0 ) {
	king_moves[pos] |= bb_square[CR2Square( x+1, y-1 )];
      }
      if( y < MAXROW-1 ) {
	king_moves[pos] |= bb_square[CR2Square( x+1, y+1 )];
      }
    }
    if( y > 0 ) {
      king_moves[pos] |= bb_square[CR2Square( x, y-1 )];
    }
    if( y < MAXROW-1 ) {
      king_moves[pos] |= bb_square[CR2Square( x, y+1 )];
    }
  }
}


// G E N E R A T I O N

// add move to moves_set[]
inline void AddMove( int set, move_t m, move_t *moves_set[] )
{
  assert( moves_set[set+1] < &moves_set[0][MAX_MOVES-1] );
  assert( IsSquareOk( Src( m ) ) );
  assert( IsSquareOk( Dst( m ) ) );
  *moves_set[set+1]++ = m;
}

inline void AddMovePromote( int set, move_t m, move_t *moves_set[] )
{
  AddMove( set, AddPromote( m, QUEEN ), moves_set );
  AddMove( set, AddPromote( m, KNIGHT ), moves_set );
  AddMove( set, AddPromote( m, ROOK ), moves_set );
  AddMove( set, AddPromote( m, BISHOP ), moves_set );
}

// generate non-capture only moves
// NOTE: 'cause this function is called after GenCaptures() we don't need to reset
//       moves_set[] pointer.
// NOTE 2: we don't make promoting moves here (GenCaptures() makes them)
void GenNonCaptures( const Board &b, int set, move_t *moves_set[] )
{
  assert( bb_init );
  assert( moves_set[set] <= moves_set[set+1] );
  
  int player = b.side;
  bitboard_t bb_pieces = b.getBBPieces();
  square_t src, dst;
  
  bitboard_t bb_moves;

  // pawn moves

  bitboard_t bb_dbl_pawn_moves;
  int src_diff;
  
  if( player == WHITE ) {
    bb_moves = ( b.getBBPieces( WHITE, PAWN ) << MAXCOL ) & ~( bb_pieces | bb_row[7] );
    bb_dbl_pawn_moves = ( ( bb_moves & bb_row[2] ) << MAXCOL ) & ~bb_pieces;
    src_diff = -MAXCOL;
  } else {
    bb_moves = ( b.getBBPieces( BLACK, PAWN ) >> MAXCOL ) & ~( bb_pieces | bb_row[0] );
    bb_dbl_pawn_moves = ( ( bb_moves & bb_row[5] ) >> MAXCOL ) & ~bb_pieces;
    src_diff = MAXCOL;
  }

  while( bb_moves ) {
    dst = LastBit( bb_moves );
    ToggleBit( bb_moves, dst );
    
    src = dst + src_diff;
    AddMove( set, MakeMove( src, dst ), moves_set );
  }

  src_diff *= 2;
  while( bb_dbl_pawn_moves ) {
    dst = LastBit( bb_dbl_pawn_moves );
    ToggleBit( bb_dbl_pawn_moves, dst );
    
    src = dst + src_diff;
    AddMove( set, MakeMove( src, dst ) | MOVE_DOUBLE_PAWN_PUSH, moves_set );
  }

  // knight moves

  bitboard_t bb_cycle = b.getBBPieces( player, KNIGHT );
  while( bb_cycle ) {
    src = LastBit( bb_cycle );
    ToggleBit( bb_cycle, src );

    bb_moves = knight_moves[src] & ~bb_pieces;
    while( bb_moves ) {
      dst = LastBit( bb_moves );
      ToggleBit( bb_moves, dst );

      AddMove( set, MakeMove( src, dst ), moves_set );
    }
  }

  // bishop moves

  bb_cycle = b.getBBPieces( player, BISHOP ) | b.getBBPieces( player, QUEEN );
  while( bb_cycle ) {
    src = LastBit( bb_cycle );
    ToggleBit( bb_cycle, src );

    bb_moves = ( bishop_moves[src][0][( int )( ( b.bb_rot45 >> rot45_shift[src] ) & rot45_and[src] )]
		 | bishop_moves[src][1][( int )( ( b.bb_rot315 >> rot315_shift[src] ) & rot315_and[src] )] ) & ~bb_pieces;
    while( bb_moves ) {
      dst = LastBit( bb_moves );
      ToggleBit( bb_moves, dst );

      AddMove( set, MakeMove( src, dst ), moves_set );
    }
  }

  // rook moves
  
  bb_cycle = b.getBBPieces( player, ROOK ) | b.getBBPieces( player, QUEEN );
  int row, col;
  const bitboard_t mask = ( bitboard_t )0xFF;
  while( bb_cycle ) {
    src = LastBit( bb_cycle );
    ToggleBit( bb_cycle, src );
    
    row = src / MAXCOL;
    col = rot90_lookup[src] / MAXCOL;

    bb_moves = ( rook_moves[src][0][( int )( ( bb_pieces >> (row*MAXCOL) ) & mask )]
		 | rook_moves[src][1][( int )( ( b.bb_rot90 >> (col*MAXCOL) ) & mask )] ) & ~bb_pieces;

    while( bb_moves ) {
      dst = LastBit( bb_moves );
      ToggleBit( bb_moves, dst );

      AddMove( set, MakeMove( src, dst ), moves_set );
    }
  }

  // king moves

  src = b.getKingPos( player );
  bb_moves = king_moves[src] & ~bb_pieces;
  while( bb_moves ) {
    dst = LastBit( bb_moves );
    ToggleBit( bb_moves, dst );

    AddMove( set, MakeMove( src, dst ), moves_set );
  }

  // castling

  const square_t castling_king_side_move[2] = { G1, G8 };
  const square_t castling_queen_side_move[2] = { C1, C8 };
  const square_t castling_king_side_attacks[2] = { F1, F8 };
  const square_t castling_queen_side_attacks[2] = { D1, D8 };

  int xside = ChangeSide( b.side );
  if( ( b.getCastlingStatus( player ) & CASTLING_KING_SIDE )
      && ( ( bb_castling[player][CASTLING_KING_SIDE] & bb_pieces ) == NULLBOARD )
      && !b.attacks( xside, castling_king_side_attacks[player] ) ) {
    AddMove( set, MakeMove( src, castling_king_side_move[player] ) | MOVE_CASTLING, moves_set );
  }
  if( ( b.getCastlingStatus( player ) & CASTLING_QUEEN_SIDE )
      && ( ( bb_castling[player][CASTLING_QUEEN_SIDE] & bb_pieces ) == NULLBOARD )
      && !b.attacks( xside, castling_queen_side_attacks[player] ) ) {
    AddMove( set, MakeMove( src, castling_queen_side_move[player] ) | MOVE_CASTLING, moves_set );
  }
}

// Generate all capture and promote moves
void GenCaptures( const Board &b, int set, move_t *moves_set[] )
{
  assert( bb_init );
  
  // reset moves_set[] before doing any operation
  moves_set[set+1] = moves_set[set];
  
  int player = b.side;
  int xplayer = ChangeSide( player );
  bitboard_t bb_friend_pieces = b.getBBPieces( player, EMPTY );
  bitboard_t bb_enemy_pieces = b.getBBPieces( xplayer, EMPTY );
  square_t src, dst;
  bitboard_t bb_moves;

  // pawn captures and promotes
  
  int src_diff;
  bitboard_t bb_pawns = b.getBBPieces( player, PAWN );
  bitboard_t bb_promote;

  // left captures...
  if( player == WHITE ) {
    bb_moves = ( bb_pawns << ( MAXCOL - 1 ) ) & bb_enemy_pieces & ~bb_col[MAXCOL-1];
    bb_promote = bb_moves & bb_row[MAXROW-1];
    bb_moves &= ~bb_promote;
    src_diff = -(MAXCOL-1);
  } else {
    bb_moves = ( bb_pawns >> ( MAXCOL + 1 ) ) & bb_enemy_pieces & ~bb_col[MAXCOL-1];
    bb_promote = bb_moves & bb_row[0];
    bb_moves &= ~bb_promote;
    src_diff = MAXCOL+1;
  }
  while( bb_moves ) {
    dst = LastBit( bb_moves );
    ToggleBit( bb_moves, dst );

    AddMove( set, MakeCapture( dst + src_diff, dst, GetPiece(b[dst]) ), moves_set );
  }
  while( bb_promote ) {
    dst = LastBit( bb_promote );
    ToggleBit( bb_promote, dst );

    AddMovePromote( set, MakeCapture( dst + src_diff, dst, GetPiece(b[dst]) ), moves_set );
  }

  // right captures...
  if( player == WHITE ) {
    bb_moves = ( bb_pawns << ( MAXCOL + 1 ) ) & bb_enemy_pieces & ~bb_col[0];
    bb_promote = bb_moves & bb_row[MAXROW-1];
    bb_moves &= ~bb_promote;
    src_diff = -(MAXCOL+1);
  } else {
    bb_moves = ( bb_pawns >> ( MAXCOL - 1 ) ) & bb_enemy_pieces & ~bb_col[0];
    bb_promote = bb_moves & bb_row[0];
    bb_moves &= ~bb_promote;
    src_diff = MAXCOL-1;
  }
  while( bb_moves ) {
    dst = LastBit( bb_moves );
    ToggleBit( bb_moves, dst );

    AddMove( set, MakeCapture( dst + src_diff, dst, GetPiece(b[dst]) ), moves_set );
  }
  while( bb_promote ) {
    dst = LastBit( bb_promote );
    ToggleBit( bb_promote, dst );

    AddMovePromote( set, MakeCapture( dst + src_diff, dst, GetPiece(b[dst]) ), moves_set );
  }

  // en passant
  if( b.en_passant != -1 ) {
    bb_moves = bb_pawns & pawn_captures[xplayer][b.en_passant];
    while( bb_moves ) {
      src = LastBit( bb_moves );
      ToggleBit( bb_moves, src );
      
      AddMove( set, MakeCapture( src, b.en_passant, PAWN ) | MOVE_EN_PASSANT, moves_set );
    }
  }

  // promotes...
  if( player == WHITE ) {
    bb_moves = ( bb_pawns << MAXCOL ) & ~bb_enemy_pieces & ~bb_friend_pieces  & bb_row[MAXROW-1];
    src_diff = -MAXCOL;
  } else {
    bb_moves = ( bb_pawns >> MAXCOL ) & ~bb_enemy_pieces & ~bb_friend_pieces & bb_row[0];
    src_diff = MAXCOL;
  }
  while( bb_moves ) {
    dst = LastBit( bb_moves );
    ToggleBit( bb_moves, dst );

    AddMovePromote( set, MakeMove( dst + src_diff, dst ), moves_set );
  }


  // knight moves

  bitboard_t bb_cycle = b.getBBPieces( player, KNIGHT );
  while( bb_cycle ) {
    src = LastBit( bb_cycle );
    ToggleBit( bb_cycle, src );

    bb_moves = knight_moves[src] & bb_enemy_pieces;
    while( bb_moves ) {
      dst = LastBit( bb_moves );
      ToggleBit( bb_moves, dst );

      AddMove( set, MakeCapture( src, dst, GetPiece(b[dst]) ), moves_set );
    }
  }

  // bishop moves

  bb_cycle = b.getBBPieces( player, BISHOP ) | b.getBBPieces( player, QUEEN );
  while( bb_cycle ) {
    src = LastBit( bb_cycle );
    ToggleBit( bb_cycle, src );

    bb_moves = ( bishop_moves[src][0][( int )( ( b.bb_rot45 >> rot45_shift[src] ) & rot45_and[src] )]
		 | bishop_moves[src][1][( int )( ( b.bb_rot315 >> rot315_shift[src] ) & rot315_and[src] )] ) & bb_enemy_pieces;
    while( bb_moves ) {
      dst = LastBit( bb_moves );
      ToggleBit( bb_moves, dst );

      AddMove( set, MakeCapture( src, dst, GetPiece(b[dst]) ), moves_set );
    }
  }

  // rook moves
  
  bb_cycle = b.getBBPieces( player, ROOK ) | b.getBBPieces( player, QUEEN );
  const bitboard_t mask = ( bitboard_t )0xFF;
  int row, col;
  while( bb_cycle ) {
    src = LastBit( bb_cycle );
    ToggleBit( bb_cycle, src );

    row = src / MAXCOL;
    col = rot90_lookup[src] / MAXCOL;
    
    bb_moves = ( rook_moves[src][0][( int )( ( ( bb_enemy_pieces | bb_friend_pieces ) >> (row*MAXCOL) ) & mask )]
		 | rook_moves[src][1][( int )( ( b.bb_rot90 >> (col*MAXCOL) ) & mask )] ) & bb_enemy_pieces;

    while( bb_moves ) {
      dst = LastBit( bb_moves );
      ToggleBit( bb_moves, dst );

      AddMove( set, MakeCapture( src, dst, GetPiece(b[dst]) ), moves_set );
    }
  }

  // king moves

  src = b.getKingPos( player );
  bb_moves = king_moves[src] & bb_enemy_pieces;
  while( bb_moves ) {
    dst = LastBit( bb_moves );
    ToggleBit( bb_moves, dst );

    AddMove( set, MakeCapture( src, dst, GetPiece(b[dst]) ), moves_set );
  }

}

// Generate all evasion moves
// The moves generated by this function are *legal*
// NOTE: this function must be called when king is in check
void GenEvasions( const Board &b, int set, move_t *moves_set[] )
{
  assert( bb_init );
  assert( b.isInCheck() );

  moves_set[set+1] = moves_set[set];

  int player = b.side;
  int xplayer = ChangeSide( player );
  square_t sqking = b.getKingPos( player );
  square_t dst;
  bitboard_t bb_attacks = b.piecesAttack( xplayer, sqking );
  assert( bb_attacks );
  bitboard_t bb_friend_pieces = b.getBBPieces( player, EMPTY );

  square_t atk_src = LastBit( bb_attacks );
  ToggleBit( bb_attacks, atk_src );
  bitboard_t bb_invalid_moves = bb_ray[atk_src][sqking] & ~BBSquare( atk_src );

  piece_t piece = GetPiece( b[atk_src] );
  if( piece != PAWN && piece != KNIGHT ) {
    if( square_next[atk_src][sqking] != SQUARE_NULL ) {
      assert( IsSquareOk( square_next[atk_src][sqking] ) );
      SetBit( bb_invalid_moves, square_next[atk_src][sqking] );
    }
  }

  bitboard_t bb_moves;
  bool king_only = false;
  
  // 2 pieces attacks, only king moves
  if( bb_attacks ) {
    atk_src = LastBit( bb_attacks );
    ToggleBit( bb_attacks, atk_src );
    assert( !bb_attacks );

    bb_invalid_moves |= bb_ray[atk_src][sqking] & ~BBSquare( atk_src );
    piece = GetPiece( b[atk_src] );
    if( piece != PAWN && piece != KNIGHT ) {
      if( square_next[atk_src][sqking] != SQUARE_NULL ) {
	assert( IsSquareOk( square_next[atk_src][sqking] ) );
	SetBit( bb_invalid_moves, square_next[atk_src][sqking] );
      }
    }
    king_only = true;
  }


  // king evasion moves
  
  bb_moves = king_moves[sqking] & ~bb_invalid_moves & ~bb_friend_pieces;
  while( bb_moves ) {
    dst = LastBit( bb_moves );
    ToggleBit( bb_moves, dst );
    if( !b.attacks( xplayer, dst ) ) {
      if( b[dst] ) {
	AddMove( set, MakeCapture( sqking, dst, GetPiece(b[dst]) ), moves_set );
      } else {
	AddMove( set, MakeMove( sqking, dst ), moves_set );
      }
    }
  }

  // if double attack, skip other moves
  if( king_only ) {
    return;
  }

  
  bitboard_t bb_attacks_ray = ( bb_ray[atk_src][sqking] & ~BBSquare( sqking ) ) & ~BBSquare( atk_src );
  bitboard_t bb_pieces = b.getBBPieces( xplayer, EMPTY ) | bb_friend_pieces;

  // pawn moves
  
  square_t src_diff, src;
  square_t promote_row;

  // single moves
  if( player == WHITE ) {
    bb_moves = ( b.getBBPieces( WHITE, PAWN ) << MAXCOL ) & bb_attacks_ray;
    src_diff = -MAXCOL;
    promote_row = MAXROW-1;
  } else {
    bb_moves = ( b.getBBPieces( BLACK, PAWN ) >> MAXCOL ) & bb_attacks_ray;
    src_diff = MAXCOL;
    promote_row = 0;
  }
  while( bb_moves ) {
    dst = LastBit( bb_moves );
    ToggleBit( bb_moves, dst );

    if( !b.pinned( player, dst+src_diff ) ) {
      if( Row( dst ) == promote_row ) {
	AddMovePromote( set, MakeMove( dst+src_diff, dst ), moves_set );
      } else {
	AddMove( set, MakeMove( dst+src_diff, dst ), moves_set );
      }
    }
  }

  // double moves
  if( player == WHITE ) {
    bb_moves = ( ( ( ( b.getBBPieces( WHITE, PAWN ) & bb_row[1] ) << MAXCOL )
		   & ~bb_pieces ) << MAXCOL ) & bb_attacks_ray;
  } else {
    bb_moves = ( ( ( ( b.getBBPieces( BLACK, PAWN ) & bb_row[MAXROW-2] ) >> MAXCOL )
		   & ~bb_pieces ) >> MAXCOL ) & bb_attacks_ray;
  }
  while( bb_moves ) {
    dst = LastBit( bb_moves );
    ToggleBit( bb_moves, dst );

    if( !b.pinned( player, dst+(src_diff*2) ) ) {
      AddMove( set, MakeMove( dst+(src_diff*2), dst ) | MOVE_DOUBLE_PAWN_PUSH, moves_set );
    }
  }

  // en passant captures: if en passant cover enemy attack ray OR
  //                      if we can capture attack piece with en passant (attack piece must be a pawn)
  if( b.en_passant != -1 && ( BBSquare( b.en_passant ) & bb_attacks_ray )
      || ( (b.en_passant+src_diff) == atk_src && piece == PAWN ) ) {
    if( player == WHITE ) {
      bb_moves = pawn_captures[BLACK][b.en_passant] & b.getBBPieces( WHITE, PAWN );
    } else {
      bb_moves = pawn_captures[WHITE][b.en_passant] & b.getBBPieces( BLACK, PAWN );
    }
    while( bb_moves ) {
      src = LastBit( bb_moves );
      ToggleBit( bb_moves, src );

      if( !b.pinned( player, src ) ) {
	AddMove( set, MakeCapture( src, b.en_passant, PAWN ) | MOVE_EN_PASSANT, moves_set );
      }
    }
  }

  // captures
  bb_moves = pawn_captures[xplayer][atk_src] & b.getBBPieces( player, PAWN );
  while( bb_moves ) {
    src = LastBit( bb_moves );
    ToggleBit( bb_moves, src );

    if( !b.pinned( player, src ) ) {
      if( Row( atk_src ) == promote_row ) {
	AddMovePromote( set, MakeCapture( src, atk_src, piece ), moves_set );
      } else {
	AddMove( set, MakeCapture( src, atk_src, piece ), moves_set );
      }
    }
  }
  

  // knight moves
  
  bb_attacks_ray |= BBSquare( atk_src ); // add capture move
  bitboard_t bb_cycle = b.getBBPieces( player, KNIGHT );
  while( bb_cycle ) {
    src = LastBit( bb_cycle );
    ToggleBit( bb_cycle, src );

    if( !b.pinned( player, src ) ) {
      bb_moves = knight_moves[src] & bb_attacks_ray;
      while( bb_moves ) {
	dst = LastBit( bb_moves );
	ToggleBit( bb_moves, dst );

	if( dst == atk_src ) {
	  AddMove( set, MakeCapture( src, dst, piece ), moves_set );
	} else {
	  AddMove( set, MakeMove( src, dst ), moves_set );
	}
      }
    }
  }


  // bishop moves
  
  bb_cycle = b.getBBPieces( player, BISHOP ) | b.getBBPieces( player, QUEEN );
  while( bb_cycle ) {
    src = LastBit( bb_cycle );
    ToggleBit( bb_cycle, src );

    if( !b.pinned( player, src ) ) {
      bb_moves = ( bishop_moves[src][0][( int )( ( b.bb_rot45 >> rot45_shift[src] ) & rot45_and[src] )]
		   | bishop_moves[src][1][( int )( ( b.bb_rot315 >> rot315_shift[src] ) & rot315_and[src] )] ) & bb_attacks_ray;
      while( bb_moves ) {
	dst = LastBit( bb_moves );
	ToggleBit( bb_moves, dst );

	if( dst == atk_src ) {
	  AddMove( set, MakeCapture( src, dst, piece ), moves_set );
	} else {
	  AddMove( set, MakeMove( src, dst ), moves_set );
	}
      }
    }
  }


  // rook moves
  
  bb_cycle = b.getBBPieces( player, ROOK ) | b.getBBPieces( player, QUEEN );
  const bitboard_t mask = ( bitboard_t )0xFF;
  int row, col;
  while( bb_cycle ) {
    src = LastBit( bb_cycle );
    ToggleBit( bb_cycle, src );

    if( !b.pinned( player, src ) ) {
      row = src / MAXCOL;
      col = rot90_lookup[src] / MAXCOL;
    
      bb_moves = ( rook_moves[src][0][( int )( ( bb_pieces >> (row*MAXCOL) ) & mask )]
		   | rook_moves[src][1][( int )( ( b.bb_rot90 >> (col*MAXCOL) ) & mask )] ) & bb_attacks_ray;

      while( bb_moves ) {
	dst = LastBit( bb_moves );
	ToggleBit( bb_moves, dst );

	if( dst == atk_src ) {
	  AddMove( set, MakeCapture( src, dst, piece ), moves_set );
	} else {
	  AddMove( set, MakeMove( src, dst ), moves_set );
	}
      }
    }
  }
}

// erase all illegal moves from a set
void EraseInvalidMoves( Board &b, int set, move_t *moves_set[] )
{
  move_t *mt = moves_set[set];
  bool legal;

  while( mt != moves_set[set+1] ) {
    assert( mt < moves_set[set+1] );

    b.makeMove( *mt );
    legal = b.isLegal();
    b.undoMove();
    
    if( !legal ) {
      *mt = *( moves_set[set+1] - 1 );
      --moves_set[set+1];
    } else {
      ++mt;
    }
  }
}
