/*
  smash, a chess engine
  Copyright (c) 2003-2006  Maurizio Sambati
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  
  See LICENSE file for more info.

  written by Maurizio Sambati <sambati@cli.di.unipi.it>
  http://www.cli.di.unipi.it/~sambati
*/
#ifndef _MOVEHANDLER_H_
#define _MOVEHANDLER_H_

/*
  MoveHandler class

  Created in 28/12/2005
  Maurizio
*/

#include "move.h"
#include "thread.h"


const int MH_ALL = 0;
const int MH_TACTICALS = 1;
const int MH_CAPTURES = 2;

enum {
  MH_PHASE_HASH,
  MH_PHASE_CAPTURES_GEN,
  MH_PHASE_CAPTURES,
  MH_PHASE_OTHERS_GEN,
  MH_PHASE_OTHERS,
  MH_PHASE_END,
  MH_PHASE_RESET_HASH,
  MH_PHASE_RESET_MOVES
};

class MoveHandler
{
 public:
  MoveHandler( ThreadData &td, int ply, move_t hash_move, int filter = MH_ALL );
  ~MoveHandler() {}

  move_t nextMove();
  const bool &singlemove;
  void reset();

 private:
  ThreadData &m_td;
  
  int m_ply;
  int m_phase;
  int m_filter;
  move_t m_hash_move;
  bool m_singlemove;
  move_t *m;
  char m_hanged[64];

  void sortCaptures();
  void sortNonCaptures();
  void sortEvasions();
  move_t findNextMove();

  bool checkFilter( move_t move );
};

void InitMoveHandler( ThreadData &td );
void FixMoveHandler( ThreadData &td, int ply );


#endif
