/*
  smash, a chess engine
  Copyright (c) 2003-2006  Maurizio Sambati
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  
  See LICENSE file for more info.

  written by Maurizio Sambati <sambati@cli.di.unipi.it>
  http://www.cli.di.unipi.it/~sambati
*/
#include <string>
#include <vector>
#include <map>
#include <algorithm>
#include <iostream>

#include "options.h"

using namespace std;


Options options;


Options::Options()
{
}

/*
  add option with minimum and maximum integer value
*/
void Options::addOption( const string &name, const string &type, const string &def,
			 int vmin, int vmax, Options::callback func )
{
  OptDesc &desc = options[name];
  desc.name = name;
  desc.type = type;
  desc.def = def;
  desc.value = def;
  desc.vmin = vmin;
  desc.vmax = vmax;
  desc.func = func;
}

/*
  add standard option
*/
void Options::addOption( const string &name, const string &type, const string &def, Options::callback func )
{
  OptDesc &desc = options[name];
  desc.name = name;
  desc.type = type;
  desc.def = def;
  desc.value = def;
  desc.func = func;
}

/*
  add predefined value of an option
*/
void Options::addVar( const string &name, const string &var )
{
  // add only if option doesn't have this var yet
  OptDesc &opt = options[name];
  if( find( opt.vars.begin(), opt.vars.end(), var ) == opt.vars.end() ) {
    opt.vars.push_back( var );
  }
}

/*
  set option value
*/
void Options::setOption( const string &name, const string &value )
{
  if( options.count( name ) == 0 ) {
    cerr << "Option " << name << " doesn't exist!" << endl;
    return;
  }
  OptDesc &desc = options[name];
  desc.value = value;
  if( desc.func != 0 ) {
    desc.func( *this, name, value );
  }
}


/*
  get option value methods
*/

int Options::getInt( const string &name ) const
{
  map<string, OptDesc>::const_iterator iter = options.find( name );
  if( iter != options.end() ) {
    return atoi( iter->second.value.c_str() );
  } else {
    return 0;
  }
}

string Options::getString( const string &name ) const
{
  map<string, OptDesc>::const_iterator iter = options.find( name );
  if( iter != options.end() ) {
    return iter->second.value;
  } else {
    return "";
  }
}

bool Options::getBool( const string &name ) const
{
  map<string, OptDesc>::const_iterator iter = options.find( name );
  if( iter != options.end() ) {
    return ( iter->second.value == "true" ) ? true : false;
  } else {
    return false;
  }
}


void Options::printDesc( ostream &out ) const
{
  for( map<string, OptDesc>::const_iterator iter = options.begin();
       iter != options.end(); ++iter ) {
    const OptDesc &opt( iter->second );
    out << "option name " << opt.name << " type " << opt.type;
    if( opt.def != "" ) {
      out << " default " << opt.def;
    }
    if( opt.type == "spin" ) {
      out << " min " << opt.vmin << " max " << opt.vmax;
    }
    if( opt.type == "combo" ) {
      for( vector<string>::const_iterator var = opt.vars.begin();
	   var != opt.vars.end(); ++var ) {
	out << " var " << *var;
      }
    }
    out << endl;
  }
}
