/*
  smash, a chess engine
  Copyright (c) 2003-2006  Maurizio Sambati
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  
  See LICENSE file for more info.

  written by Maurizio Sambati <sambati@cli.di.unipi.it>
  http://www.cli.di.unipi.it/~sambati
*/
#ifndef _OPTIONS_H_
#define _OPTIONS_H_

/*
  engine options

  Created in 19/10/2005
  Maurizio
*/

#include <string>
#include <vector>
#include <map>
#include <iostream>


// O P T I O N S   C L A S S

class Options
{
 public:
  Options();

  typedef bool (*callback)( Options &opt, const std::string &name, const std::string &value );

  void addOption( const std::string &name, const std::string &type, const std::string &def, int vmin,
		  int vmax, callback func );
  void addOption( const std::string &name, const std::string &type, const std::string &def, callback func );
  void addVar( const std::string &name, const std::string &var );
  
  void setOption( const std::string &name, const std::string &value );
  
  int getInt( const std::string &name ) const;
  std::string getString( const std::string &name ) const;
  bool getBool( const std::string &name ) const;

  void printDesc( std::ostream &out ) const;

 private:
  struct OptDesc {
    std::string name, type, def;
    int vmin, vmax;
    std::vector<std::string> vars;
    std::string value;
    callback func;
  };
  std::map<std::string, OptDesc> options;
};

extern Options options;


#endif

