/*
  smash, a chess engine
  Copyright (c) 2003-2006  Maurizio Sambati
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  
  See LICENSE file for more info.

  written by Maurizio Sambati <sambati@cli.di.unipi.it>
  http://www.cli.di.unipi.it/~sambati
*/
#include "perft.h"
#include "movegen.h"
#include "board.h"
#include "thread.h"
#include "search.h"

#include <string>
#include <iostream>

using namespace std;

static uint64_t Perft( Board &b, int depth, int ply );

// return perft raw nodes at specified depth
uint64_t Perft( Board &b, int depth )
{
  if( depth == 0 ) {
    return 1;
  } else {
    return Perft( b, depth, 0 );
  }
}

// perform perft recursively
uint64_t Perft( Board &b, int depth, int ply )
{
  ThreadData &td( thread_data[0] );
  
  bool check = b.setCheckStatus();

  if( !check ) {
    GenCaptures( b, ply, td.moves_set );
    GenNonCaptures( b, ply, td.moves_set );
  } else {
    GenEvasions( b, ply, td.moves_set );
  }

  uint64_t count = 0;
  for( move_t *m = td.moves_set[ply]; m < td.moves_set[ply+1]; ++m ) {
    b.makeMove( *m );
    if( check || b.isLegal() ) {
      if( depth == 1 ) {
	++count;
      } else {
	count += Perft( b, depth-1, ply+1 );
      }
    }
    b.undoMove();
  }
  return count;
}
