/*
  smash, a chess engine
  Copyright (c) 2003-2006  Maurizio Sambati
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  
  See LICENSE file for more info.

  written by Maurizio Sambati <sambati@cli.di.unipi.it>
  http://www.cli.di.unipi.it/~sambati
*/
/*
  Recognizer module
  An implementation of Heinz's Recognizers

  Created in 13/02/2006
  Maurizio
*/

#include "recognizer.h"
#include "ttable.h"
#include "board.h"


// recognize easy material draw for the specified side
inline bool EasyMaterialDraw( const Board &board, int side )
{
  if( board.getNumPieces( side, PAWN ) + board.getNumPieces( side, QUEEN )
      + board.getNumPieces( side, ROOK ) > 0 ) {
    return false;
  } else if( board.getNumPieces( side, BISHOP ) + board.getNumPieces( side, KNIGHT ) == 1 ) {
    return true;
  } else if( board.getNumPieces( side, KNIGHT ) == 0
	     && ( !( board.getBBPieces( side, BISHOP ) && bb_white_squares )
		  || !( board.getBBPieces( side, BISHOP ) && bb_black_squares ) ) ) {
    return true;
  } else {
    return false;
  }
}

// recognize some material draw quickly
bool FastDrawRecognize( const Board &board, RecognizerScore &result )
{
  // rule 50
  if( board.rule50 >= 100 ) {
    result.score = 0;
    result.scoretype = SCORE_EXACT;
    return true;
  }

  // K vs K
  if( board.getNumPieces() == 2 ) {
    result.score = 0;
    result.scoretype = SCORE_EXACT;
    return true;
  }

  // easy material draw
  if( EasyMaterialDraw( board, board.side ) ) {
    result.score = 0;
    if( EasyMaterialDraw( board, ChangeSide( board.side ) ) ) {
      result.scoretype = SCORE_EXACT;
    } else {
      result.scoretype = SCORE_FAILLOW;
    }
    return true;
  } else if( EasyMaterialDraw( board, ChangeSide( board.side ) ) ) {
    result.score = 0;
    result.scoretype = SCORE_FAILHIGH;
    return true;
  }

  return false;
}
