/*
  smash, a chess engine
  Copyright (c) 2003-2006  Maurizio Sambati
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  
  See LICENSE file for more info.

  written by Maurizio Sambati <sambati@cli.di.unipi.it>
  http://www.cli.di.unipi.it/~sambati
*/
#ifndef _SMP_H_
#define _SMP_H_

/*
  SMP threads, locks and utility functions

  Created in 20/03/2006
  Maurizio
*/

#include "history.h"
#include "killer.h"
#include "move.h"
#include "movegen.h"

#include <algorithm>

// L O C K

// From Crafty... (with minimal changes)

#if defined(USESMP)

#define VOLATILE volatile

// change this if you wanna try with different number of threads
// NOTE: you must change also the value of NumThreads option
const int MAX_THREADS = 4;

#if defined(_WIN32) || defined(_WIN64)

#include <windows.h>

   typedef HANDLE thread_t;
   typedef HANDLE event_t;

#define THREADRET DWORD WINAPI
inline bool NewThread( thread_t &tid, THREADRET (*func)( void *), void *arg )
{
  tid = CreateThread( NULL, 0, func, arg, 0, NULL );
  return tid != NULL;
}

inline void EventWait( event_t &event )
{
  WaitForSingleObject( event, INFINITE );
}

inline void EventNotifyAll( event_t &event )
{
  SetEvent( event );
}

inline void EventInit( event_t &event )
{
  event = CreateEvent( 0, TRUE, FALSE, 0 );
}

inline void EventReset( event_t &event )
{
  ResetEvent( event );
}



#  if defined(_M_ALPHA) && !defined(NT_INTEREX)

      extern "C" __int64 __asm (char *, ...);
      extern "C" void __MB (void);
#     pragma intrinsic (__asm)
#     pragma intrinsic (__MB)

      typedef volatile int lock_t[1];

inline void LockInit( lock_t v )
{
  v[0] = 0;
}

inline void LockFree( lock_t v )
{
  v[0] = 0;
}

inline void Unlock( lock_t v )
{
  __MB();
  v[0] = 0;
}

inline void Lock( lock_t hPtr )
{
  __asm( "lp: ldl_l v0,(a0);"
	 "    xor v0,1,v0;"
	 "    beq v0,lp;"
	 "    stl_c v0,(a0);"
	 "    beq v0,lp;"
	 "    mb;",
	 hPtr );
}

#  elif (defined (_M_IX86) && !defined(NT_INTEREX))

     typedef volatile int lock_t[1];

inline void LockInit( lock_t v )
{
  v[0] = 0;
}

inline void LockFree( lock_t v )
{
  v[0] = 0;
}

inline void Unlock( lock_t v )
{
  v[0] = 0;
}


#    if (_MSC_VER > 1200 || 1)

#include <iostream>

inline void Lock( lock_t hPtr )
{
  int iValue;
  volatile int *hPtrTmp;

  hPtrTmp = hPtr;     // Workaround for vc7 beta1 bug
  iValue = 1;
  while( 1 )
    {
      iValue = InterlockedExchange( (long *)hPtrTmp, iValue );
      if( 0 == iValue ) {
	break;
      }
      while( *hPtrTmp )
	;   // Do nothing
    }
}

#  else

inline void Lock( volatile int *hPtr )
{
      __asm
	{
	  mov     ecx, hPtr
	    la: mov     eax, 1
	    xchg    eax, [ecx]
	    test    eax, eax
	    jz      end
	    lb:  mov     eax, [ecx]
	    test    eax, eax
	    jz      la
	    jmp     lb
	    end:
	}
}

#  endif

#elif defined (_M_IA64) && !defined(NT_INTEREX)

#  include <windows.h>

#  pragma intrinsic (_InterlockedExchange)

   typedef volatile LONG lock_t[1];

inline void LockInit( lock_t v )
{
  v[0] = 0;
}

inline void LockFree( lock_t v )
{
  v[0] = 0;
}

inline void Unlock( lock_t v )
{
  v[0] = 0;
}

inline void Lock( lock_t hPtr )
{
  int iValue;

  for (;;)
    {
      iValue = _InterlockedExchange( static_cast<LPLONG>( hPtr ), 1 );
      if (0 == iValue)
	return;
      while( hPtr[0] )
	;   // Do nothing
    }
}


#else /* NT non-Alpha/Intel, without assembler Lock() */

typedef volatile int lock_t;
inline void LockInit( lock_t &v )
{
  v = 0;
}

inline void LockFree( lock_t &v )
{
  v = 0;
}

inline void Lock( lock_t &v )
{
  while( InterlockedExchange( static_cast<LPLONG>(&v), 1 ) != 0 )
    ;
}

inline void Unlock( lock_t &v )
{
  v = 0;
}

#endif /* architecture check */

#else  /* not NT, assume SMP using POSIX threads (LINUX, etc)  */

#include <pthread.h>

#if defined(MUTEX)

typedef pthread_mutex_t lock_t;

inline void Lock( lock_t &v )
{
  pthread_mutex_lock( &v );
}

inline void LockInit( lock_t &v )
{
  pthread_mutex_init( &v, 0 );
}

inline void LockFree( lock_t &v )
{
  pthread_mutex_destroy( &v );
}

inline void Unlock( lock_t &v )
{
  pthread_mutex_unlock( &v );
}

#elif defined(ALPHA)

#  if defined(__DECC)
#    include <machine/builtins.h>
#  endif

typedef volatile long lock_t;

inline void LockInit( lock_t &v )
{
  v = 0;
}

inline void LockFree( lock_t &v )
{
  v = 0;
}

inline void Lock( lock_t &v )
{
  __LOCK_LONG( &v );
}

inline void Unlock( lock_t &v )
{
  __UNLOCK_LONG( &v );
}

#else /* POSIX, but not using MUTEXes */

#  define exchange(adr,reg)                                  \
     ({ volatile int _ret;                                   \
     asm volatile ("xchg %0,%1"                              \
     : "=q" (_ret), "=m" (*(adr))    /* Output %0,%1 */      \
     : "m"  (*(adr)), "0"  (reg));   /* Input (%2),%0 */     \
     _ret;                                                   \
     })

typedef volatile int lock_t;

inline void LockInit( lock_t &v )
{
  v = 0;
}

inline void LockFree( lock_t &v )
{
  v = 0;
}

inline void Lock( lock_t &v )
{
  while( exchange( &v, 1 ) ) {
    while( v );
  }
}

inline void Unlock( lock_t &v )
{
  exchange( &v, 0 );
}

#endif /* MUTEX */

   typedef pthread_t thread_t;
   typedef struct {
     pthread_cond_t cond;
     pthread_mutex_t mutex;
   } event_t;

#define THREADRET void *
inline bool NewThread( thread_t &tid, THREADRET (*func)( void *), void *arg )
{
  return pthread_create( &tid, NULL, func, arg ) == 0;
}

inline void EventWait( event_t &event )
{
  pthread_mutex_lock( &event.mutex );
  pthread_cond_wait( &event.cond, &event.mutex );
  pthread_mutex_unlock( &event.mutex );
}

inline void EventNotifyAll( event_t &event )
{
  pthread_cond_broadcast( &event.cond );
}

inline void EventInit( event_t &event )
{
  pthread_cond_init( &event.cond, NULL );
  pthread_mutex_init( &event.mutex, NULL );
}

#  define EventReset( e )


#endif


#else

#define VOLATILE

const int MAX_THREADS = 1;

typedef int lock_t;
typedef int thread_t;
typedef int event_t;

#  define LockInit( p )
#  define LockFree( p )
#  define Lock( p )
#  define Unlock( p )
#  define EventInit( e )
#  define EventReset( e )
#  define EventWait( e )
#  define EventNotifyAll( e )

#  define THREADRET void *

inline bool NewThread( thread_t &tid, THREADRET (*func)(void *), void *arg )
{
  return false;
}

#endif

// C L A S S E S

/*
  Locker class:
    -- you don't need to call Unlock()
*/
class Locker
{
public:
  Locker( lock_t &l ) : lock( l ) { Lock( lock ); }
  ~Locker() { Unlock( lock ); }

private:
  lock_t &lock;
};


/*
  AtomicNum<T> is a simple class to implement atomic number
*/
template<class T>
class AtomicNum
{
 public:
  AtomicNum() { LockInit( lock ); }
  AtomicNum( T n ) : num( n ) { LockInit( lock ); }

  // volatile operators
  // TODO: some operations can be atomic without use explicit Lock/Unlock
  T operator++() volatile { T ret; Lock( lock ); ret = ++num; Unlock( lock ); return ret; }
  T operator--() volatile { T ret; Lock( lock ); ret = --num; Unlock( lock ); return ret; }
  T operator++( int ) volatile { T ret; Lock( lock ); ret = num++; Unlock( lock ); return ret; }
  T operator--( int ) volatile { T ret; Lock( lock ); ret = num--; Unlock( lock ); return ret; }
  T operator+=( T v ) volatile { T ret; Lock( lock ); ret = num += v; Unlock( lock ); return ret; }
  T operator-=( T v ) volatile { T ret; Lock( lock ); ret = num -= v; Unlock( lock ); return ret; }
  T operator=( T v ) volatile { Lock( lock ); num = v; Unlock( lock ); return v; }
  
  T maximize( T v ) volatile { T ret; Lock( lock ); ret = num = std::max( const_cast<T&>(num), v ); Unlock( lock ); return ret; }
  T minimize( T v ) volatile { T ret; Lock( lock ); ret = num = std::min( const_cast<T&>(num), v ); Unlock( lock ); return ret; }

  // non-volatile operators
  T operator++() { return ++num; }
  T operator--() { return --num; }
  T operator++( int ) { return num++; }
  T operator--( int ) { return num--; }
  T operator+=( T v ) { return num += v; }
  T operator-=( T v ) { return num -= v; }
  T operator=( T v ) { return num = v; }

  // const operations
  T get() VOLATILE const { return num; }
  operator T() VOLATILE const { return num; }

  bool operator<( T v ) VOLATILE const  { return num < v; }
  bool operator<=( T v ) VOLATILE const { return num <= v; }
  bool operator==( T v ) VOLATILE const { return num == v; }
  bool operator>=( T v ) VOLATILE const { return num >= v; }
  bool operator>( T v ) VOLATILE const  { return num > v; }
  bool operator!=( T v ) VOLATILE const { return num != v; }

 private:
  T num;
  lock_t lock;
};

#endif
