/*
  smash, a chess engine
  Copyright (c) 2003-2006  Maurizio Sambati
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  
  See LICENSE file for more info.

  written by Maurizio Sambati <sambati@cli.di.unipi.it>
  http://www.cli.di.unipi.it/~sambati
*/
#include "stat.h"
#include "search.h"

#include <cstdio>

using namespace std;

// S T A T I S T I C S   M E T H O D S

// reset statistics
void SearchStatistics::reset()
{
  NodesCnt = FailHigh = FailLow = ExactScore = LeafNodes = 0;
  QNodes = QFailHigh = QFailLow = QExactScore = QStandPat = 0;
  QDeltaPruning = 0;
  TTCuts = QTTCuts = TTBadEntries = 0;
  NullMoves = NullCuts = 0;
  CutFirstMove = CutSecondMove = 0;
  CheckExt = OneReplyExt = PawnEndingExt = 0;
}

// dump statistics on log
void SearchStatistics::dump()
{
  uint64_t SearchNodes = FailHigh + FailLow + ExactScore;

  printf( "info string Nodes: " U64F ", Quiesce: " U64F " (%d%%)\n", NodesCnt, QNodes, PERC( QNodes, NodesCnt ) );
  printf( "info string FailHigh: " U64F " (%d%%), FailLow: " U64F " (%d%%), ExactScore: " U64F " (%d%%)\n", FailHigh,
	  PERC( FailHigh, SearchNodes ), FailLow, PERC( FailLow, SearchNodes ), ExactScore, PERC( ExactScore, SearchNodes ) );
  printf( "info string CutFirstMove: " U64F " (%d%%), CutSecondMove: " U64F " (%d%%)\n", CutFirstMove, PERC( CutFirstMove, FailHigh ),
	  CutSecondMove, PERC( CutSecondMove, FailHigh ) );
  printf( "info string TTCuts: " U64F " (%d%%) QTTCuts: " U64F " (%d%%) TTBadEntries: " U64F "\n", TTCuts, PERC( TTCuts, NodesCnt ),
	  QTTCuts, PERC( QTTCuts, QNodes ), TTBadEntries );
  printf( "info string NullMoves: " U64F " NullCuts: " U64F " (%d%%)\n", NullMoves, NullCuts, PERC( NullCuts, NullMoves ) );
  printf( "info string Quiesce StandPat: " U64F " (%d%%) FailHigh: " U64F " (%d%%) FailLow: " U64F " (%d%%) ExactScore " U64F " (%d%%)\n",
	  QStandPat, PERC( QStandPat, QNodes ), QFailHigh, PERC( QFailHigh, QNodes ),
	  QFailLow, PERC( QFailLow, QNodes ), QExactScore, PERC( QExactScore, QNodes ) );
  printf( "info string DeltaPruning: " U64F " (%d%%)\n", QDeltaPruning, PERC( QDeltaPruning, QNodes+QDeltaPruning ) );
  printf( "info string CheckExt: " U64F " OneReplyExt: " U64F " PawnEndingExt: " U64F "\n", CheckExt, OneReplyExt, PawnEndingExt );
  ttable.dumpInfo();

  fflush( stdout );
}

