/*
  smash, a chess engine
  Copyright (c) 2003-2006  Maurizio Sambati
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  
  See LICENSE file for more info.

  written by Maurizio Sambati <sambati@cli.di.unipi.it>
  http://www.cli.di.unipi.it/~sambati
*/
#include <iostream>
#include <string>
#include <algorithm>
#include <cctype>
#include "smash.h"
#include "board.h"
#include "move.h"
#include "gametree.h"
#include "movegen.h"
#include "perft.h"
#include "timecontrol.h"
#include "search.h"
#include "see.h"

using namespace std;

static void Test();

int main()
{
  Initialize();
  Test();
}


static void Test()
{
#ifdef TEST
  TestSEE();
  TestBoard();
#else
  cerr << "TEST UNDECLARED!!" << endl;
#endif
}


// utility functions

move_t FlipMove( move_t move )
{
  square_t src, dst;
  src = Src( move );
  dst = Dst( move );
  src = FlipSquare( src );
  dst = FlipSquare( dst );

  return AddMove( move & ~(MOVE_SQUARE_MASK|(MOVE_SQUARE_MASK<<MOVE_DEST_SHIFT)), src, dst );
}

// flip the board
string FlipFen( const string &fen )
{
  int pos = fen.find( ' ' );
  assert( pos != string::npos );
  string board = fen.substr( 0, pos );
  string rotatedboard;

  // reverse rows
  pos = board.rfind( '/' );
  int prev = string::npos;
  assert( pos != string::npos );
  do {
    assert( prev == string::npos || prev > pos );
    rotatedboard +=  board.substr( pos + 1, prev == string::npos ? string::npos : prev - (pos+1) );
    rotatedboard += "/";

    assert( pos > 0 );
    prev = pos;
    pos = board.rfind( '/', pos-1 );
  } while( pos != string::npos );

  assert( prev > 0 && prev != string::npos );
  rotatedboard += board.substr( 0, prev );

  // replace lower-cased letters with upper one and vice-versa
  // this is done to flip the color of each piece
  for( int i = 0; i < rotatedboard.length(); ++i ) {
    if( islower( rotatedboard[i] ) ) {
      rotatedboard[i] = toupper( rotatedboard[i] );
    } else if( isupper( rotatedboard[i] ) ) {
      rotatedboard[i] = tolower( rotatedboard[i] );
    }
  }

  pos = fen.find( ' ' );
  assert( pos != string::npos );
  rotatedboard += fen.substr( pos );

  pos = rotatedboard.find( ' ' );
  assert( pos != string::npos );
  ++pos;

  // change side to move
  assert( rotatedboard[pos] == 'w' || rotatedboard[pos] == 'b' );
  rotatedboard[pos] = rotatedboard[pos] == 'w' ? 'b' : 'w';

  // change castling
  // NOTE: swapcase and sort is enough
  pos += 2;
  char ch = rotatedboard[pos];
  assert( ch == '-' || ch == 'K' || ch == 'k' || ch == 'q' || ch == 'Q' );
  prev = pos;
  do {
    if( islower( rotatedboard[pos] ) ) {
      rotatedboard[pos] = toupper( rotatedboard[pos] );
    } else if( isupper( rotatedboard[pos] ) ) {
      rotatedboard[pos] = tolower( rotatedboard[pos] );
    }
    ++pos;
  } while( rotatedboard[pos] != ' ' );

  sort( rotatedboard.begin() + prev, rotatedboard.begin() + pos );

  // en passant
  ++pos;
  ch = rotatedboard[pos];
  assert( ch == '-' || ( ch >= 'a' && ch <= 'h' ) );
  if( rotatedboard[pos] >= 'a' && rotatedboard[pos] <= 'h' ) {
    ++pos;
    assert( isdigit( rotatedboard[pos] ) );
    if( rotatedboard[pos] == '3' ) {
      rotatedboard[pos] = '6';
    } else if( rotatedboard[pos] == '6' ) {
      rotatedboard[pos] = '3';
    } else {
      assert( 0 );
    }
  }

  return rotatedboard;
}
