/*
  smash, a chess engine
  Copyright (c) 2003-2006  Maurizio Sambati
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  
  See LICENSE file for more info.

  written by Maurizio Sambati <sambati@cli.di.unipi.it>
  http://www.cli.di.unipi.it/~sambati
*/
#include <algorithm>

#include "smash.h"
#include "timecontrol.h"
#include "search.h"

using namespace std;

/*
  start timer
*/
void TimeControl::start()
{
  stopped = false;
  if( infinite ) {
    return;
  }

  starttime = GetTime();

  int inc = thread_data[0].board.side == WHITE ? winc : binc;
  int currtime = thread_data[0].board.side == WHITE ? wtime : btime;
  
  if( fixedtime == -1 ) {
    if( currtime != -1 ) {
      int remain;
      if( movestogo != -1 ) {
	remain = movestogo;
	if( inc != -1 ) {
	  remain = max( 1, remain - 1 );
	}
      } else {
	remain = 35;
      }
      maxtime = currtime / remain;
      if( maxtime <= 100 ) {
	maxtime = max( currtime/2, 2 );
      }
      mintime = maxtime / 2;

      if( remain == 1 ) {
	maxtime = currtime - 100;
	mintime = maxtime / 2;
      }
      maxtime /= 10;
      mintime /= 10;
    } else {
      infinite = true;
    }
  } else {
    maxtime = mintime = fixedtime / 10;
  }
}

void TimeControl::stop()
{
  stopped = true;
}

/*
  return true if time up
*/
bool TimeControl::check()
{
  int elapsedtime = GetTime() - starttime;
  if( elapsedtime < 0 ) {
    // TODO: underflow
    elapsedtime += starttime;
  }

  return stopped || ( !infinite && ( elapsedtime >= maxtime ) );
}

/*
  return true if there is more time to compute next ply
*/
bool TimeControl::nextPly()
{
  int elapsedtime = GetTime() - starttime;

  if( elapsedtime < 0 ) {
    // TODO: underflow
    elapsedtime += starttime;
  }

  return !( stopped || ( !infinite && ( elapsedtime >= mintime ) ) );
}
