/*
  smash, a chess engine
  Copyright (c) 2003-2006  Maurizio Sambati
  
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  
  See LICENSE file for more info.

  written by Maurizio Sambati <sambati@cli.di.unipi.it>
  http://www.cli.di.unipi.it/~sambati
*/
#ifndef _TIMECONTROL_H_
#define _TIMECONTROL_H_

/*
  smash timecontrol module

  Created in 1/10/2005
  Maurizio
*/

#if defined(_WIN32) || defined(_WIN64)
#include <windows.h>
#else
#include <sys/time.h>
#include <sys/ioctl.h>
#include <sys/stat.h>
#include <sys/types.h>
#endif

#include "thread.h"

// get centiseconds
inline unsigned int GetTime()
{
#if defined(_WIN32) || defined(_WIN64)
  return GetTickCount()/10;
#else
  timeval tv;
  gettimeofday( &tv, 0 );
  return ( tv.tv_sec * 100 + ( tv.tv_usec / 10000 ) );
#endif
}

class TimeControl
{
 public:
  int wtime, btime, winc, binc;
  int movestogo;
  int fixedtime;
  bool infinite;

  void start();
  void stop();
  bool check();
  bool nextPly();

 private:
  bool stopped;
  int starttime;
  int maxtime, mintime;
};

class TimeException : public ThreadInterrupt
{
 public:
  TimeException() throw() {}
  virtual ~TimeException() throw() {}

  virtual const char *what() const throw() {
    return "time exception";
  }
};

#endif
